unit IDEPlugClass;

interface
uses
  Windows,
  SysUtils,
  Classes,
  Controls,
  Messages,
  uIDE,
  System.Reflection,
  System.Runtime.InteropServices,
  uSharedNET;


{$UNSAFECODE ON}

type

    [Guid('0DF824AB-4F99-4E5F-917D-848179A3C377'),
	 ClassInterface(ClassInterfaceType.AutoDual)]
  TIDEClass = class(MarshalByRefObject)
  private
    strNewState : string;
    FModuleFolder: string;
    procedure SetModuleFolder(const Value: string);
  public
    constructor Create;

  function CompileTimeBuild(MSIHandle: uint; const State, BuildPath: String; Build: Boolean;
  var Fetch: Boolean): Boolean;
    function DesignTimeEdit(Window: uint; const State: String; var NewState: String;
      DisplayDialog : Boolean): Integer;
    function DesignTimeText(const State: string; var Text: string): Integer;
    procedure AboutPlugIn(Window: uint);

    property ModuleFolder : string read FModuleFolder write SetModuleFolder;
  end;

implementation





procedure TIDEClass.AboutPlugIn(Window: uint);

{
  This function is called during Design Time
  Window: Handle to parent window
  See help for details
}
begin
  MessageBox(Window, 'InstallAware Template Delphi Plug-In', 'About Plug-In',
    mb_Ok + mb_IconInformation);
end;

function TIDEClass.CompileTimeBuild(MSIHandle: uint; const State, BuildPath: String; Build: Boolean;
  var Fetch: Boolean): Boolean;
{
  This function is called during Compile Time
  MSIHandle: Handle to MSI file being built
  State: State of plug-in controls, as provided to the IDE previously
  BuildPath: Output folder for build that plug-in should use
  Build: Whether any plug-in and/or support files should actually be copied
  Fetch: Whether plug-in will require any files in addition to the runtime DLL
  Return: Set to TRUE if compile succeeds, FALSE if it fails
  See help for details
}
  var
  l: TStringList;
  Option1: Boolean;
  Variable: String;
begin
  Result := True; // compile time build successful
  l := TStringList.Create;
  l.CommaText := State;
  Option1 := l[0] = 'TRUE';
  Fetch := not Option1; // assume option 2 requires some files to be added to build media

  Variable := l[1];
  l.Free;

  if Build then
  begin
    // add files to build media, if necessary
    if not Option1 then
      CopyFile( ModuleFolder + '2copy\example.txt',
        BuildPath + 'example.txt', false);

    // copy runtime DLL
    CopyFile(ModuleFolder + 'mEXEPlug.dll',
      BuildPath + 'mEXEPlug.dll', false);
                                  
    // copy NET Bridge
    CopyFile(ModuleFolder + 'IDEPlug.dproj_NetBridge.dll',
      BuildPath + 'IDEPlug.dproj_NetBridge.dll', false);

    // copy NET Bridge Config file
    CopyFile(ModuleFolder + 'IDEPlug.dproj_NetBridge.ini',
      BuildPath + 'IDEPlug.dproj_NetBridge.ini', false);
  end;
end;

constructor TIDEClass.Create;
begin
  inherited Create;
end;



function TIDEClass.DesignTimeEdit(Window: uint; const State: String; var NewState: String;
  DisplayDialog : Boolean): Integer;
  {
  This function is called during Design Time
  Window: Handle to parent window
  State: State of plug-in controls, as provided to the IDE previously
  NewState: New state of plug-in controls, as edited by user
  See help for details
}
var
  l: TStringList; // string list to hold state of user selections
begin
  if DisplayDialog then
  begin
    { this call is to display the user interface }
    IDEPlugIn := TIDEPlugIn.Create(nil);

    l := TStringList.Create;
    if State = '' then
    // default values enter here
    begin
      l.Add('FALSE');
      l.Add('');
    end else
    l.CommaText := State;

    with IDEPlugIn do
    begin
      // initialize values of variables
      Option1.Checked := l[0] = 'TRUE';
      Option2.Checked := not Option1.Checked;
      Variable.Text := l[1];

      // display dialog
      if ShowModal = mrOk then
      begin
        // save values of variables
        if Option1.Checked then l[0] := 'TRUE'
          else l[0] := 'FALSE';

        l[1] := Variable.Text;
        // strNewState holds state of user selections passed back to IDE
        strNewState := l.CommaText;
        Result := Length(strNewState);
      end       else
        Result := -1;
    end;
    l.Free;
    IDEPlugIn.Free;



  end else begin
    { this call is to obtain user selection state }
  NewState := strNewState;
  Result := 0;
  end;

end;

function TIDEClass.DesignTimeText(const State: string; var Text: string): Integer;
{
  This function is called during Design Time
  State: State of plug-in controls, as provided to the IDE previously
  Text: Plug-in state represented textually in the IDE script view
  See help for details
}
var
  l: TStringList;
begin
  l := TStringList.Create;
  l.CommaText := State;
  strNewState := ': ';
  if l[0] = 'TRUE' then
    strNewState := strNewState + 'Do something'
  else
    strNewState := strNewState + 'Do something else';
  if l[1] <> '' then
    strNewState := strNewState + ' (get result into variable ' + l[1] + ')';
  l.Free;
  Result := Length(strNewState);
  Text := strNewState;end;
procedure TIDEClass.SetModuleFolder(const Value: string);
begin
  FModuleFolder := Value;
end;

end.
