library IDEPlug.dproj_NetBridge;

uses
  windows,
  SysUtils,
  Controls,
  Classes,
  uIDE in 'uIDE.pas' {IDEPlugIn},
  uSharedDLL in 'uSharedDLL.pas',
  DotNetWrapper in 'DotNetWrapper.pas';




{
  This function is called during Compile Time
  MSIHandle: Handle to MSI file being built
  State: State of plug-in controls, as provided to the IDE previously
  BuildPath: Output folder for build that plug-in should use
  Build: Whether any plug-in and/or support files should actually be copied
  Fetch: Whether plug-in will require any files in addition to the runtime DLL
  Return: Set to TRUE if compile succeeds, FALSE if it fails
  See help for details
}
function CompileTimeBuild(MSIHandle: THandle; State, BuildPath: PChar; Build: Boolean;
  var Fetch: Boolean): Boolean; stdcall; export;
var
  ModuleFolder, sState, sBuildPath : string;
begin
  initWrapper;
  ModuleFolder := ExtractFilePath(GetCurrentDLLName);
  Wrapper.Assembly.ModuleFolder := ModuleFolder;

  sState := State;
  sBuildPath := BuildPath;

  result := Wrapper.Assembly.CompileTimeBuild(
    uint(MSIHandle),
    sState, sBuildPath,
    Build,
  Fetch);
end;

{
  This function is called during Design Time
  Window: Handle to parent window
  State: State of plug-in controls, as provided to the IDE previously
  NewState: New state of plug-in controls, as edited by user
  See help for details
}
function DesignTimeEdit(Window: THandle; State: PChar; NewState: PChar): Integer; stdcall; export;
var
  sState, sNewState : string;
begin
  InitWrapper;
  sState := State;
  sNewState := NewState;


  result := Wrapper.Assembly.DesignTimeEdit(
    uint(window),
    sState, sNewState,
    (NewState = NIL));



  if NewState <> nil then
    StrCopy(NewState, PChar(sNewState));
end;

{
  This function is called during Design Time
  State: State of plug-in controls, as provided to the IDE previously
  Text: Plug-in state represented textually in the IDE script view
  See help for details
}
function DesignTimeText(State, Text: PChar): Integer; stdcall; export;
var
  sText, sState : string;
begin
  InitWrapper;
  sText := Text;
  sState := State;



  result := Wrapper.Assembly.DesignTimeText(sState, sText);


  if Text = nil then
  begin
    { this call is to get length of the user readable text }
  // nothing to do
  end
  else
  begin
    { this call is to obtain the actual user readable text }
    StrCopy(Text, PChar(sText));
    Result := 0;
  end;
end;

{
  This function is called during Design Time
  Window: Handle to parent window
  See help for details
}
procedure AboutPlugIn(Window: THandle); stdcall; export
begin
  InitWrapper;
  Wrapper.Assembly.AboutPlugIn(uint(Window));

end;

exports
  CompileTimeBuild,
  DesignTimeEdit,
  DesignTimeText,
  AboutPlugIn;







begin
  GetMem(p, 1024);
  GetModuleFileName(GetModuleHandle('mIDEPlug.dll'), p, 1024);
  ModuleFile := p;
  ModuleFolder := ExtractFilePath(p);
  FreeMem(p);
end.
