{ ============================================================================== }
{ Copyright(c) 1996-2009 InstallAware Software Corporation. All rights reserved. }
{ This source code provided for demonstrative use only. All rights reserved.     }
{ ============================================================================== }

unit uSharedDLL;

interface
uses
  SysUtils, Windows, Classes, DotNetWrapper;


var
  Wrapper : TDotNetWrapper = NIL;
var
  ModuleFile, ModuleFolder: String; // global variables used for storing the plug-in DLL and folder location
  cacheVariables: String; // stores the list of script variables

type
  TPlugProgressIndicator = function(SmartPosition: Integer; Text: PChar): Boolean; stdcall; // progress callback for 
  //runtime

{ Use to replace variables used in strings with their values }
function SubstituteVariables(Literal: String; Variables: TStrings): String;

{ Call to get the value of a variable }
function GetVariable(Variables: TStrings; Variable: String): String;
{ Call to set the value of a variable }
procedure SetVariable(Variables: TStrings; Variable: String; Value: String);


function ReadAssemblyCfg(out AssemblyFilename, nameSpace : string): Boolean;
procedure InitWrapper;
procedure FreeWrapper;
function GetCurrentDLLName : string;

implementation
uses
  iniFiles;

function GetVariable(Variables: TStrings; Variable: String): String;
var
  i: Integer;
begin
  for i := 1 to Variables.Count do
    if ((i -1) mod 2) = 0 then // do we really have a variable at hand?
      if AnsiUpperCase(Variables[i -1]) = AnsiUpperCase(Variable) then
      begin
        Result := Variables[i];
        Exit;
      end;
  raise Exception.Create('Attempting to get value of undefined variable ' + Variable);
end;

procedure SetVariable(Variables: TStrings; Variable: String; Value: String);
var
  i: Integer;
begin
  for i := 1 to Variables.Count do
    if ((i -1) mod 2) = 0 then // do we really have a variable at hand?
      if AnsiUpperCase(Variables[i -1]) = AnsiUpperCase(Variable) then
      begin
        Variables[i] := Value;
        Exit;
      end;
  raise Exception.Create('Attempting to set value of undefined variable ' + Variable);
end;

function SubstituteVariables(Literal: String; Variables: TStrings): String;
var
  i: Integer;
begin
  Result := Literal;
  for i := 1 to Variables.Count do
    if (i -1) mod 2 = 0 then
      Result := StringReplace(Result, '$' + Variables[i -1] + '$',
        Variables[i], [rfReplaceAll, rfIgnoreCase]);
  if Pos('$', Result) <> 0 then
    if Literal <> Result then
      Result := SubstituteVariables(Result, Variables);
end;



function GetCurrentDLLName : string;
var
  s : string;
  i : integer;
begin
  SetLength(s, MAX_PATH);
  i := GetModuleFileName(hinstance, @s[1], MAX_PATH);
  if i > 0 then
  begin
    SetLength(s, i);
    Result := s;
  end else
    result := '';

end;


function ReadAssemblyCfg(out AssemblyFilename, nameSpace : string): Boolean;
var
  ini: TiniFile;
  fn : string;
begin
  result := false;
  fn := GetCurrentDLLName;
  fn := ChangeFileExt(fn, '.ini');
  if FileExists(fn) then
  begin
    ini := TiniFile.Create(fn);
    AssemblyFilename := ini.ReadString('CFG', 'FILE', '');
    AssemblyFilename:= ExtractFilePath(fn) + AssemblyFilename;
    NameSpace := ini.ReadString('CFG', 'NAMESPACE', '');
    ini.Free;

    Result := FileExists(AssemblyFilename);
  end;


end;



procedure InitWrapper;
var
  AssemblyFileName, NameSpace : string;
begin
  if not Assigned(Wrapper) then
  begin
    // initialize the wrapper
    if not ReadAssemblyCfg(AssemblyFileName, NameSpace) then
      Raise Exception.Create('Can not load plugin dll');
    Wrapper := TDotNetWrapper.Create(
      AssemblyFileName, NameSpace);
  end;
end;

procedure FreeWrapper;
begin
  if Assigned(Wrapper) then
  begin
    Wrapper.Free;
    Wrapper := NIL;
  end;
end;




end.
