unit NetRegistrationToolMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, mscorlib_TLB, Buttons, ActiveX, COMObj, ComCtrls, ExtCtrls;

type

  TPlugProgressIndicator = function(SmartPosition: Integer; Text: PChar): Boolean; stdcall;

   TRunTimeExecute = function(Window: THandle; Variables, State: PChar; Progress: TPlugProgressIndicator;
  var Return: Integer; NewVariables: PChar): Integer; stdcall;


  TForm1 = class(TForm)
    edLog: TMemo;
    Panel1: TGroupBox;
    lbProgress: TLabel;
    ProgressBar1: TProgressBar;
    PageControl1: TPageControl;
    tsRunTime: TTabSheet;
    tsIDE: TTabSheet;
    btnTestCallback: TButton;
    btnAboutPlugIn: TBitBtn;
    btnDesignTimeText: TBitBtn;
    btnDesignTimeEdit: TBitBtn;
    btnCompileTimeBuild: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure btnTestCallbackClick(Sender: TObject);
    procedure btnAboutPlugInClick(Sender: TObject);
    procedure btnDesignTimeTextClick(Sender: TObject);
    procedure btnDesignTimeEditClick(Sender: TObject);
    procedure btnCompileTimeBuildClick(Sender: TObject);
  private
    function GetRunTimeBridgeFilename : string;
    function GetIDEBridgeFilename : string;
    procedure Log(const msg: string);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
uses
  bsUTILS, jclSysInfo, jclDotNet{, pawel1}, iniFiles;


const
  CR = #13 + #10;

{$R *.dfm}




function ProgressCallback(SmartPosition: Integer; const Text: Pchar): Boolean; stdcall;
Begin
  {the progress callback, this will be called from the .net assembly}
  Form1.lbProgress.Caption := text;
  Form1.ProgressBar1.Position := SmartPosition;
    Application.ProcessMessages;
End;



function TForm1.GetIDEBridgeFilename: string;
var
  ini: TiniFile;
begin
  ini := TiniFile.Create(ChangeFileExt(Application.exeName, '.ini'));;
  result := GetInstalldir + ini.ReadString('CFG', 'IDE_BRIDGEFILENAME', '');
  ini.Free;
end;

function TForm1.GetRunTimeBridgeFilename : string;
var
  ini: TiniFile;
begin
  ini := TiniFile.Create(ChangeFileExt(Application.exeName, '.ini'));;
  result := GetInstalldir + ini.ReadString('CFG', 'RUNTIME_BRIDGE_FILENAME', '');
  ini.Free;
end;


procedure TForm1.Log(const msg: string);
begin
  edLog.Lines.Add(msg);
  edlog.Perform(WM_VSCROLL, SB_BOTTOM, 0);
end;

procedure TForm1.btnAboutPlugInClick(Sender: TObject);
Const
  ProcName = 'AboutPlugIn';
Type
  TProc = procedure (Window: THandle); stdcall;
var
  Lib : THandle;
  Proc : TProc;
  fn : string;
begin
  fn := GetIDEBridgeFilename;
Log('Loading Bridge DLL From : '+fn);
  lib := windows.LoadLibrary(pChar(fn));
  @proc := GetProcAddress(Lib,ProcName);

  Log('Calling '+ProcName);
  Proc(self.Handle);



  // release the Bridge dll
  windows.FreeLibrary(lib);
end;


Function b2s(b : Boolean) : string;
begin
  if b then
    Result := 'true'
  else
    Result := 'False';

end;

procedure TForm1.btnCompileTimeBuildClick(Sender: TObject);
Const
  ProcName = 'CompileTimeBuild';
Type
  TProc = function (MSIHandle: THandle; State, BuildPath: PChar; Build: Boolean;
  var Fetch: Boolean): Boolean; stdcall;
var
  Lib : THandle;
  Proc : TProc;
  fn : string;
  BuildPath : string;
  bResult : boolean;
  Fetch : Boolean;
  State : string;
begin
  fn := GetIDEBridgeFilename;
Log('Loading Bridge DLL From : '+fn);
  lib := windows.LoadLibrary(pChar(fn));
  @proc := GetProcAddress(Lib,ProcName);


  BuildPath := GetInstalldir + 'Build\';
  State := 'TRUE,MyVar';
  Forcedirectories(BuildPath);

  Log('Calling '+ProcName);
  bResult := Proc(0, pChar(State), pChar(BuildPath), True, Fetch);
  Log('  Result: '+b2s(bResult)+CR+'  Fetch: '+b2s(Fetch));
                                                        

  // release the Bridge dll
  windows.FreeLibrary(lib);
end;

procedure TForm1.btnDesignTimeEditClick(Sender: TObject);
Const
  ProcName = 'DesignTimeEdit';
Type
  TProc = function (Window: THandle; State: PChar; NewState: PChar): Integer; stdcall; 
var
  s: string;
  iResult: integer;
  Lib : THandle;
  Proc : TProc;
  fn : string;
begin
  fn := GetIDEBridgeFilename;
Log('Loading Bridge DLL From : '+fn);
  lib := windows.LoadLibrary(pChar(fn));
  @proc := GetProcAddress(Lib,ProcName);

  Log('Calling '+ProcName+' - Display Dialog');
  iResult := Proc(self.Handle, nil, nil);
  Log('  Result: '+IntToStr(iResult));

  if iresult > 0 then
  begin
    SetLength(s, iResult);
    Log('Calling '+ProcName+' - get NewStates');
    iResult := Proc(self.Handle, nil, @s[1]);
    Log('  Result: '+IntToStr(iResult)+CR+
      '  NewStates: '+s);
  end;


  // release the Bridge dll
  windows.FreeLibrary(lib);

end;

procedure TForm1.btnDesignTimeTextClick(Sender: TObject);
Type
  TProc = function (State, Text: PChar): Integer; stdcall;
  Const
    ProcName = 'DesignTimeText';
var
Text,   State: string;
  iResult: integer;
  Lib : THandle;
  Proc : TProc;
  fn : string;
begin
  fn := GetIDEBridgeFilename;
Log('Loading Bridge DLL From : '+fn);
  lib := windows.LoadLibrary(pChar(fn));
  @proc := GetProcAddress(Lib,ProcName);

  // init variables
  State := 'TRUE,,FALSE';
  Text := '';

  Log('Calling '+ProcName+' - to get Text Length');
  Log('  in Params: '+CR+'    State: '+State);
  iResult := Proc(pChar(State), nil);
  Log('  Out Params: '+CR+'    Text: '+Text+CR+'    Result: '+IntToStr(iResult));;

  SetLength(Text, iResult);
  Log('Calling '+ProcName+' - to get Text Value');
  Log('  in Params: '+CR+'    State: '+State);
  iResult := Proc(pChar(State), pChar(Text));
  Log('  Out Params: '+CR+'    Text: '+Text+CR+'    Result: '+IntToStr(iResult));




  // release the Bridge dll
  windows.FreeLibrary(lib)
end;

procedure TForm1.btnTestCallbackClick(Sender: TObject);
const
  cDomain = 'DemoLib';

var
  fn: string;
  NewVariables: string;
  iReturn: integer;
  Lib : THandle;
  RunTimeExecute :    TRunTimeExecute;
  iRes : integer;
begin
  fn := GetRunTimeBridgeFilename;


Log('Loading Bridge DLL From : '+fn);
  lib := windows.LoadLibrary(pChar(fn));
  @RunTimeExecute := GetProcAddress(Lib,'RunTimeExecute');


    //set input values
    iReturn := -1;
    SetLength(NewVariables, 1024);



    //call the method
    iRes := RunTimeExecute(
      self.handle,
      'MyVariables', 'MyState',
    @ProgressCallback,
    iReturn,
    @NewVariables[1]);


    // Log Output Values
    Log(' RunTimeExecute was successfuly called'+CR+
      '  Result: '+IntToStr(iRes)+CR+
      '  Return: '+intToStr(iReturn)+CR+
      '  NewVariables: "'+NewVariables+'"');




  // release the Bridge dll
  windows.FreeLibrary(lib)
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  edLog.lines.clear;
end;

{ TProgObj }




INITIALIZATION
  // When working with .NET from Delphi, it is important calling Set8087CW($133F); at the begining of your program (
  //i.e. before Application.Initialize;). Delphi has enabled floating point exceptions by default and the
  //CLR doesnt like them. When I had them enabled, my program weirdly freezed.

  Set8087CW($133F);
end.
