 {************************************************************}
 {  bsutils.pas                                               }
 {  (c)1998/1999 Business Software                            }
 {  http://www.ebsoft.freeserve.co.uk/                        }
 { Collection of String / System / Math utility classes       }
 {         for Borland Delphi(r) Programmers.                 }

 { Author: Eddie Bond ebinfo@compuserve.com                   }
 {************************************************************}

 // FileName:            bsutils2.zip
 // Program:    BSUTILS collection of Pascal utility classes
 // Ver:      2.1
 // Date:    30 October 1999
 // Copyright:    (c)1999 Business Software
 // Web:      http://www.bsoft.ic24.net/
 // Author:    Eddie Bond
 // E-Mail:    ebinfo@compuserve.com
 // Status:    FreeWare
 // Restrictions:  None.
 // Delphi:    32-bit versions
 // Platform:    Windows 32-bit versions.
 // Distribution:  Freely distribute ENTIRE package.

 // NB.
 // This source code is distributed by Business Software as FREEWARE
 // with the author's permission.
 // IT IS NOT PUBLIC DOMAIN!
 // You may use these utilities in your applications, whether private
 // or commercial, without payment or royalties.
 // You may distribute this file in unadulterated and unmodified form,
 // or include this file together, or as part of your own distributed
 // project's source code provided that this header, and all comments
 // remain attached and readable.
 // you may 'cut and paste' program segments from this file, to incorporate
 // into your own projects, but if you publish the source code you should
 // show the following comment below the program segment:

  {from bsutils.pas
  (c)1999 Business Software http://www.bsoft.ic24.net}

 // NB This code is provided without warranty or support of any kind. You use
 // this code entirely at your own risk.

{*************************************************************}

 { CHECK OUT OUR SITE http://www.bsoft.ic24.net/       }
 { DELPHI FREEWARE - SHAREWARE - ADVICE - DOWNLOADS            }

{*************************************************************}

{===============================================================}


unit bsutils;

{$B-,H+}

interface


uses Windows, Messages, SysUtils, Forms, Classes, registry, DB,{dbtables,BDE,}shellapi;

{================= String Utils =================}

function slash(Value: string): string;
 {ensures that value has '\' as last character (for directory strings)}
 {example: slash('c:\My Documents') = 'c:\My Documents\'}
 {example: slash('D:\') = 'D:\' }

function capfirst(Value: string): string;
 {Capitalise first character of each word, lowercase remaining chars}
 {example: capfirst('bOrLANd delPHi FOR windOWs') = 'Borland Delphi For Windows'}

function striptags(Value: string): string;
 {strip HTML tags from value}
 {example: striptags('<TR><TD Align="center">Hello World</TD>') = 'Hello World'}

function striphtmidents(Value: string): string;
{strip common html identifiers (&nbsp, &reg, &copy) from value}

function replace(const str, s1, s2: string; casesensitive: boolean): string;
 {replace all incidences of s1 in str with s2}
 {example: replace('We know what we want','we','I',false) = 'I Know what I want'}

function replacechar(s: string; c: char; i: integer): string;
 {replace character at position i in string s with character c}
 {example: replacechar('charactar','e',8) = 'character}

function CopyFromChar(s: string; c: char; l: integer): string;
 {copy l characters from string s starting at first incidence of c}
 {example: Copyfromchar('Borland Delphi','a',3) = 'and'}

procedure splitstring(const s: string; delim: char; ts: TStrings);
{splits a compound string (s) delimited by (delim) into TStrings (ts)}
{example SplitString('Tony,Blair,Downing Street,London',',',ts) returns:
ts.count=4
ts[0]='Tony'
ts[1]='Blair'
ts[2]='Downing Street'
ts[3]='London'}

function posnext(s, srch: string; lastpos: integer): integer;
 {repeatedly search for position of srch within s starting at lastpos}
 {example: to find the positions of 'ab' within s}
{s:='abcdefgsabcvderabnbvcswahgabfabxserta';
x:=1;
while x<>0 do
      begin
      x:=posnext(s,'ab',x);
      (do something with the value x here...)
      end;
}

function currstrtofloat(Value: string): single;
 {converts currency style strings to a floating point value}
 {example: currstrtofloat('$100,000.45')=100000.45}

function strtofloattry(const Value: string): extended;
 {tries to convert value to a valid floating point value.returns zero if it fails}
 {example: strtofloattry('a100d3.65')=1003.65}
 {example: strtofloattry('notthistime')=0}

function padstring(s: string; c: char; l: integer): string;
 {pads string s with as many extra characters c required to achieve length l}
 {if l is less than length of s, result is s truncated to length l}
 {example: padstring('hello','-',10)='hello-----'}
 {example: padstring('string less than l','-',15)='string less than'}

function strs(c: char; l: integer): string;
 {returns a string containing l of character c.}
 {example: strs('=',12) = '============'}


{================= System Utils =================}

function getwinsysdir: string;
 {returns Windows System Path (inc drive)}
 {example: getwinsysdir = 'C:\WINDOWS\SYSTEM\'}

function getwindir: string;
 {returns windows directory path (inc Drive)}
 {example: getwindir = 'C:\WINDOWS\'}

function getinstalldir: string;
 {returns install directory of EXE using this library}
 {example: getinstalldir = 'C:\PROGRAM FILES\BORLAND\DELPHI\DEMOS\'}

function getcurrentdir: string;
 {returns current directory}
 {example: getcurrentdir = 'D:\DELPHI PROJECTS\CLASSES\UTILS\'}

function getbootdir: string;
 {returns boot directory}
 {example: getbootdir = 'C:\'  }

function getregvalue(root: integer; key, Value: string): string;
 {reads a registry value}
 {example: getregvalue(HKEY_LOCAL_MACHINE,'network\logon\','username') = 'Eddie Bond'}

 //function getaliaspath(dbset:Tdataset):string;
 {returns DOS path of an ACTIVE dataset's (TTable or TQuery) database alias}
 {example getaliaspath(Table1) = 'C:\Program Files\Borland\Delphi\Demos\Data\'}

function GetFileDate(const FileName: string): Tdatetime;
{returns a file's date in TDateTime format}

function folderexists(const dir: string; cancreate: boolean): boolean;
{checks if folder exists. If cancreate is true, tries to create folder, and returns success}

function DiskInDrive(Drive: char): boolean;
 {checks if drive is ready without}
 {the low-level 'Not Reading Drive ?: Abort, Retry, Ignore' message}
 {example: DiskInDrive(a) = false (if the the disk drive is empty)}

function BSGetDiskFreeSpace(drv: string; var ffree, fsize: comp): boolean;
{get disk free space for BIG disks ie > 2.1GB }

{================= File Management ==================}

function filecopy(f, t: string; NoOverwrite: boolean): boolean;
 {returns success of copy from file f to t with overwrite protection}
 {example: filecopy('C:\WINDOWS\WIN.INI','C:\BACKUPS\WIN.INI',true)=true}

function AppendToFile(filename: string; ts: TStrings): boolean;
 {appends ts to the end of textfile 'filename'. if filename does not exist, it is created}
 {example: AppendToFile('c:\autoexec.bat',memo1.lines}
 {appends the contents of memo1 to autoexec.bat}

function UniqueFilename(path: string): string;
 {Generates a unique (random) filename; checks that it is unique within the givep path}
 {example: UniqueFilename(c:\) = A2W88UH7.77Y }

{================= Arithmetic Utils =================}

function StrToFloatDef(const s: string; def: extended): extended;
 {converts S into a number. If S is invalid, returns the number passed in Def.}
 {example: strtofloatdef('$10.25',0) = 0}

function VolSphere(radius: single): extended;
{volume of sphere of given radius}

function AreaSphere(radius: single): extended;
{surface area of sphere of given radius}

function VolCylinder(radius, Height: single): extended;
{volume of cylinder of given radius and height}

function AreaCylinder(radius, Height: single): extended;
{surface area of cylinder of given radius and height}

function MinExt(const A: array of extended): extended;
{returns minimum value of an array of extended}

function MaxExt(const A: array of extended): extended;
{returns maximum value of an array of extended}

function MinInteger(const A: array of integer): integer;
{returns minimum value of an array of integers}

function MaxInteger(const A: array of integer): integer;
{returns maximum value of an array of integers}

function InverseSum(const a: array of single): single;
{solves formulae of type 1/r = 1/a + 1/b +...1/n (eg electrical resistance in parallel)}

{================= Financial Utils =================}

function MarkUp(profit: single): single;
 {returns markup percentage required to return a profit of profit percent}
 {example: MarkUp(25) = 20 }

function SellingPrice(net: double; markup: single): double;
 {returns selling price after adding markup percent to net}
 {example: SellingPrice(199.50,22.5) = 244.3875}

function NetPrice(gross: double; taxrate: single): double;
 {returns the net value of an item of gross value containing tax at taxrate percent}
 {example: NetPrice(199.99,17.5) = 170.204255319149}

function Contrast(Value: integer): integer;
 {returns a contrasting color from value}
 {example: Canvas.font.color:=contrast(canvas.brush.color);}

const
  ukton_Kg   = 1016;       {UK Ton to KiloGram}
  yard_mtr   = 0.9144;     {Yard to Metre}
  acre_SqMtr = 4047;       {Acre to Square Feet}
  oz_Kg      = 0.02835;    {Ounce to KiloGram}
  acre_ha    = 0.4047;     {Acre to Hectare}
  ukgal_ltr  = 4.546;      {UK Gallon to Litre}
  usgal_ltr  = 3.7854;     {US Gallon to Litre}
  lbsqin_kpa = 6.895;      {Pounds per Square Inch to KiloPascals}
  mile_km    = 1.6093;     {Miles to Kilometres}
  in_cm      = 2.54;       {Inches to Centimetres}
  sqin_sqcm  = 6.4516;     {Square Inches to Square Centimetres}
  cuin_cucm  = 16.387;     {Cubic Inches to Cubic Centimetres}

implementation

var
  // see initialization
  GoodComa: char = '.';
  BadComa:  char = ',';


function slash(Value: string): string;
begin
  if (Value = '') then
    Result := ''
  else
  begin
    if (Value[length(Value)] <> '\') then
      Result := Value + '\'
    else
      Result := Value;
  end;
end;

function capfirst(Value: string): string;
var
  i: integer;
  s: string;
begin
  s := uppercase(Value[1]);
  for i := 2 to length(Value) do
    if (Value[i - 1] in [' ', ',', ':', ';', '.']) then
      s := s + uppercase(Value[i])
    else
      s := s + lowercase(Value[i]);
  Result := s;
end;

function striptags(Value: string): string;
var
  i: integer;
  s: string;
begin
  i := 1;
  s := '';
  while i <= length(Value) do
  begin
    if Value[i] = '<' then
      repeat
        Inc(i)
      until (Value[i] = '>')
    else
      s := s + Value[i];
    Inc(i);
  end;
  Result := s;
end;


function replace(const str, s1, s2: string; casesensitive: boolean): string;
var
  i:    integer;
  s, t: string;
begin
  s := '';
  t := str;
  repeat
    if casesensitive then
      i := pos(s1, t)
    else
      i := pos(lowercase(s1), lowercase(t));
    if i > 0 then
    begin
      s := s + Copy(t, 1, i - 1) + s2;
      t := Copy(t, i + Length(s1), MaxInt);
    end
    else
      s := s + t;
  until i <= 0;
  Result := s;
end;

function replacechar(s: string; c: char; i: integer): string;
var
  x: integer;
begin
  Result := '';
  for x := 1 to length(s) do
    if x = i then
      Result := Result + c
    else
      Result := Result + s[x];
end;

function striphtmidents(Value: string): string;
var
  s: string;
begin
  s      := replace(Value, '&nbsp;', ' ', False);
  s      := replace(s, '&reg', '(r)', False);
  s      := replace(s, '&copy', '(c)', False);
  Result := s;
end;


function CopyFromChar(s: string; c: char; l: integer): string;
var
  i: integer;
begin
  i      := pos(c, s);
  Result := copy(s, i, l);
end;

procedure SplitString(const s: string; delim: char; ts: TStrings);
var
  i: integer;
  t: string;
begin
  t := '';
  for i := 1 to length(s) do
  begin
    if (s[i] <> delim) then
      t := t + s[i]
    else
    begin
      ts.add(t);
      t := '';
    end;
  end;

  if t <> '' then
    ts.add(t);
end;

function padstring(s: string; c: char; L: integer): string;
var
  i: integer;
  t: string;
begin
  if length(s) < L then
    Result := copy(s, 1, L)
  else
  begin
    for i := 0 to L - (length(s)) - 1 do
      t := t + c;
    Result := s + t;
  end;
end;

function strs(c: char; l: integer): string;
var
  i: integer;
begin
  Result := '';
  for i := 1 to l do
    Result := Result + c;
end;

function posnext(s, srch: string; lastpos: integer): integer;
var
  t: string;
  x: integer;
begin
  t := copy(s, lastpos + length(srch), maxint);
  x := pos(srch, t);
  if x > 0 then
    Result := lastpos + x
  else
    Result := 0;
end;

function currstrtofloat(Value: string): single;
var
  e: integer;
begin
  Result := 0;
  if (Value = '') then
    exit;
  if Value[1] = currencystring then
    Value := copy(Value, 2, length(Value) - 1);
  Value := replace(Value, thousandseparator, '', False);
  val(Value, Result, e);
  if e <> 0 then
    Result := 0;
end;

function getwinsysdir: string;
var
  p: PChar;
  z: integer;
begin
  z := 255;
  getmem(p, z);
  getsystemdirectory(p, z);
  Result := slash(string(p));
  freemem(p, z);
end;

function getwindir: string;
var
  p: PChar;
  z: integer;
begin
  z := 255;
  getmem(p, z);
  getwindowsdirectory(p, z);
  Result := slash(string(p));
  freemem(p, z);
end;

function getbootdir: string;
var
  rg: Tregistry;
begin
  rg := Tregistry.Create;
  rg.rootkey := HKEY_LOCAL_MACHINE;
  try
    if rg.openkey('Software\Microsoft\Windows\CurrentVersion\Setup', False) then
    begin
      Result := slash(rg.readstring('BootDir'));
      rg.closekey;
    end;
  finally
    rg.Free;
  end;
end;

function getcurrentdir: string;
var
  p: PChar;
  z: integer;
begin
  z := 255;
  getmem(p, z);
  getcurrentdirectory(z, p);
  Result := slash(string(p));
  freemem(p, z);
end;


function getinstalldir: string;
begin
  Result := slash(extractfiledir(application.exename));
end;

function folderexists(const dir: string; cancreate: boolean): boolean;
begin
{$I-}
  chdir(dir);
  Result := (IOResult = 0);
  if cancreate then
    if not Result then
    begin
      MKDir(dir);
      chdir(dir);
      Result := (IOResult = 0);
    end;
{$I+}
end;

function DiskInDrive(Drive: char): boolean;
var
  EM: word;
begin
  Result := False;
  if Drive in ['a'..'z'] then
    Dec(Drive, $20);
  if not (Drive in ['A'..'Z']) then
    exit;
  EM := SetErrorMode(SEM_FailCriticalErrors);
  try
    Result := DiskSize(Ord(Drive) - $40) > -1;
  finally
    SetErrorMode(EM); {restore original errormode}
  end;
end;

function UniqueFilename(path: string): string;
var
  c: char;
begin
  repeat
    Result := '';
    randomize;
    repeat
      c := chr(random(43) + 47);
      if (length(Result) = 8) then
        Result := Result + '.'
      else
      if (c in ['0'..'9', 'A'..'Z']) then
        Result := Result + c;
    until length(Result) = 12;
    Result := slash(path) + Result;
  until not (fileexists(Result));
end;

function BSGetDiskFreeSpace(drv: string; var ffree, fsize: comp): boolean;
type
  LargeInt = comp;          {use INT64 for delphi versions 3+}
type
  pLargeInt = ^comp;

var
  GetDiskFreeSpaceEx: function(RootName: PChar;
    var FreeForCaller, TotNoOfBytes: LargeInt;
    TotNoOfFreeBytes: pLargeInt):
  BOOL; stdcall;

  LFree, LSize: comp;
  Lib: THandle;
begin
  Result := False;
  LFree  := -1;
  LSize  := -1;
  Lib    := GetModuleHandle('Kernel32'); {get the handle for kernel32.dll}
  if (Lib <> 0) then
  begin
    @GetDiskFreeSpaceEx := GetProcAddress(Lib, 'GetDiskFreeSpaceExA');
    if (@GetDiskFreeSpaceEx <> nil) then
      if GetDiskFreeSpaceEx(PChar(drv), LFree, LSize, nil) then
      begin
        FFree  := LFree;
        FSize  := LSize;
        Result := True;
      end;
    FreeLibrary(Lib);
  end;
end;



function getregvalue(root: integer; key, Value: string): string;
var
  rg: Tregistry;
begin
  rg := Tregistry.Create;
  try
    rg.rootkey := root;
    if rg.OpenKey(key, False) then
      Result := rg.readString(Value)
    else
      Result := '';
  finally
    rg.Free;
  end;
end;

{
function getaliaspath(dbset:Tdataset):string;
var
vDBDesc:DBDesc;
s:string;
begin
result:='';
if not (dbset.active) then exit;
if (dbset is TTable) then s:=(dbset as ttable).databasename;
if (dbset is TQuery) then s:=(dbset as tquery).databasename;
Check(DbiGetDatabaseDesc(PChar(s),@vDBDesc));
result:=slash(string(vDBDesc.szPhyName));
end;
}

function GetFileDate(const FileName: string): Tdatetime;
begin
  if fileexists(filename) then
    Result := filedatetodatetime(fileage(filename))
  else
    Result := maxint;
end;

function filecopy(f, t: string; NoOverwrite: boolean): boolean;
begin
  Result := copyfile(PChar(f), PChar(t), NoOverwrite);
end;

function AppendToFile(filename: string; ts: TStrings): boolean;
var
  f: textfile;
  i: integer;
begin
  Result := True;
  assignfile(f, filename);
  try
    if fileexists(filename) then
      append(f)
    else
      rewrite(f);
    for i := 0 to ts.Count - 1 do
      writeln(f, ts[i]);
    closefile(f);
  except
    Result := False;
  end;
end;


function strtofloatdef(const s: string; def: extended): extended;
var
  i: integer;
begin
  val(s, Result, i);
  if i <> 0 then
    Result := def;
end;


procedure DoInit;
var
  s: string;
  f: single;
begin
  s := FloatToStr(0.5);
  f := StrToFLoat(s);
  if f <> 0.5 then
    sleep(0);
  if Pos(',', s) > 0 then
  begin
    BadComa  := '.';
    GoodComa := ',';
  end
  else
  begin
    GoodComa := '.';
    BadComa  := ',';
  end;
end;


function strtofloattry(const Value: string): extended;
var
  s:      string;
  len, i: integer;
  FirstComa: boolean;
begin

  Result := 0;
  if (Value = '') then
    exit;

  s := Value;
  s := replace(s, BadComa, GoodComa, False);

  i   := 1;
  FirstComa := True;
  len := Length(s);


  while (i <= len) do
  begin
    if s[i] = GoodComa then
    begin
      if FirstComa then
      begin
        FirstComa := False;
        Inc(i);
      end
      else
      begin
        Delete(s, i, 1);
        Dec(len);
      end;
    end
    else if (s[i] in ['0'..'9']) or ((s[i] = '-') and (i = 1)) then
      Inc(i)
    else
    begin
      Delete(s, i, 1);
      Dec(len);
    end;
  end;

  Result := StrToFloat(s);
end;




function volsphere(radius: single): extended;
begin
  Result := ((4 / 3) * pi * radius * radius * radius);
end;

function areasphere(radius: single): extended;
begin
  Result := (4 * pi * radius * radius);
end;

function volcylinder(radius, Height: single): extended;
begin
  Result := (pi * radius * radius * Height);
end;

function areacylinder(radius, Height: single): extended;
begin
  Result := (2 * pi * radius * Height);
end;

function MinExt(const A: array of extended): extended;
var
  i: integer;
begin
  Result := A[Low(A)];
  for i := Low(A) + 1 to High(A) do
    if A[i] < Result then
      Result := A[I];
end;

function MaxExt(const A: array of extended): extended;
var
  i: integer;
begin
  Result := A[Low(A)];
  for i := Low(A) + 1 to High(A) do
    if A[i] > Result then
      Result := A[I];
end;

function MinInteger(const A: array of integer): integer;
var
  i: integer;
begin
  Result := A[Low(A)];
  for i := Low(A) + 1 to High(A) do
    if A[i] < Result then
      Result := A[I];
end;

function MaxInteger(const A: array of integer): integer;
var
  i: integer;
begin
  Result := A[Low(A)];
  for i := Low(A) + 1 to High(A) do
    if A[i] > Result then
      Result := A[I];
end;

function InverseSum(const a: array of single): single;
var
  i: integer;
begin
  Result := 0;
  for i := low(a) to high(a) do
    Result := Result + (1 / a[i]);
  Result := (1 / Result);
end;

function MarkUp(profit: single): single;
begin
  Result := (100 - (10000 / (100 + profit)));
end;

function SellingPrice(net: double; markup: single): double;
begin
  Result := net + (net * markup / 100);
end;

function NetPrice(gross: double; taxrate: single): double;
begin
  Result := gross - (gross * (taxrate) / (100 + taxrate));
end;

function Contrast(Value: integer): integer;
var
  r, g, b: integer;
begin
  r := Value shr 16;
  g := (Value and $FF00) shr 8;
  b := (Value and $FF);
  if r < $A0 then
    r := $FF
  else
    r := $00;
  if g < $C0 then
    g := $FF
  else
    g := $00;
  if b < $A0 then
    b := $FF
  else
    b := $00;
  Result := b + (g * $FF) + (r * $FFFF);
end;




initialization
  DoINIT;
end.

